-- TitanVPCap.lua
--

--[[
TODO:


]]

-- Constants
local ADDON_NAME = "TitanVPCap";
local VALOR_CURRENCY_ID = 396;
local JUSTICE_CURRENCY_ID = 395;
local ELDER_CHARM_CURRENCY_ID = 697;
local LESSER_CHARM_CURRENCY_ID = 738;
local MOGU_CHARM_CURRENCY_ID = 752;

local WHITE_BRIGHT_COLOR = "|cffFFFFFF";
local YELLOW_NEON_COLOR = "|cffFFFF00";
local RED_NEON_COLOR = "|cffFF0000";
local ORANGE_NEON_COLOR = "|cffEE6622";
local GREEN_NEON_COLOR = "|cff00FF00";
local END_COLOR = FONT_COLOR_CODE_CLOSE;
local IVORY_COLOR = "|cFFFF00FF";

-- list of dungeonIDs of interest to the addon
local LFR_DUNGEONIDS = {
				MVa = 527,
				MVb = 528,
				HoFa = 529,
				HoFb = 530,
				ToES = 526};
				
local LFR_STARTER_DUNGEON = 527;
				
local LFR_CELL_A_DUNGEONS = {
				MVa = 527,
				MVb = 528};

local LFR_CELL_B_DUNGEONS = {
				HoFa = 529,
				HoFb = 530};

local LFR_CELL_C_DUNGEONS = {
				ToES = 526};
				
local LFR_CELL_D_DUNGEONS = {};
				
				
local CLASS_COLOR = {
	DEATHKNIGHT = "|cFFC41F3B",
	DRUID = "|cFFFF7D0A",
	HUNTER = "|cFFABD473",
	MAGE = "|cFF69CCF0",
	MONK = "|cFF00FF96",
	PALADIN = "|cFFF58CBA",
	PRIEST = "|cFFFFFFFF",
	ROGUE = "|cFFFFF569",
	SHAMAN = "|cFF0070DE",
	WARLOCK = "|cFF9482C9",
	WARRIOR = "|cFFC79C6E"
}

local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, false)
local LSM = LibStub("LibSharedMedia-3.0")

-- Start Titan VP Cap Progress
VPCap = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0");
VPCap.version = GetAddOnMetadata(ADDON_NAME, "Version");
VPCap.category = GetAddOnMetadata(ADDON_NAME, "X-Category");
VPCap.versionMajor = 1;    -- Used for DB versioning

-- Load Libs
VPCap.aceConfig = LibStub:GetLibrary("AceConfig-3.0");
VPCap.aceConfigDialog = LibStub:GetLibrary("AceConfigDialog-3.0");
VPCap.qtip = LibStub:GetLibrary("LibQTip-1.0");
VPCap.ldb = LibStub:GetLibrary("LibDataBroker-1.1");
VPCap.ldbIcon = LibStub:GetLibrary("LibDBIcon-1.0");
VPCap.callback = LibStub("CallbackHandler-1.0"):New(VPCap);

VPCap.currentPlayer = UnitName('player');	-- name of the logged-in character.
VPCap.currentRealm = GetRealmName();		-- current Realm we're on.
VPCap.menu = false;       -- Menu frame
VPCap.menuItems = {       -- Table for the dropdown menu
    top = {},
    middle = {},
    bottom = {},
};
VPCap.periodMaxVP = -1;
VPCap.maxJP = -1;
VPCap.maxVP = -1;
VPCap.timezone = time({year=1970, month=1, day=2, hour=0})-86400;

---- LDB Object
local ldbObj = {
        type = "data source",
        --icon = "Interface\\AddOns\\TitanVPCap\\TitanVPCap.tga",
		icon = "Interface\\Icons\\pvecurrency-valor",
		iconCoords = { 0.075, 0.925, 0.075, 0.925 },
        label = L.core.buttonLabel,
        text = 'n/a',
        category = VPCap.category,
        version = VPCap.version,
};
VPCap.ldbObj = ldbObj;	

-- OnLoad
function VPCap:OnInitialize()

    -- Make sure everything is ok with the db
    if not TitanVPCap_Storage or type(TitanVPCap_Storage) ~= 'table' then TitanVPCap_Storage = {}; end
    
    -- Tell the player we are being loaded
	self:Print(format(L.core.load, self.version));
        
    -- Load settings
    self.db = LibStub("AceDB-3.0"):New("TitanVPCap_Settings", VPCap_Defaults, true);
	self:UpdateSettings();
	
	-- Set Up LDB
    
    VPCap.ldbObj = ldbObj;
	
    -- Set back to a launcher if text is off
	if not self:GetLDB() then
		ldbObj.type = 'launcher';
		ldbObj.text = nil;
	end
    
    -- Start LDB
	self.ldb = self.ldb:NewDataObject(L.core.name, ldbObj);
	self.ldbUpdated = 0;
	self.ldbLabel = '';
    
	-- Start the minimap icon
	self.ldbIcon:Register(L.core.name, self.ldb, self.db.global.minimap);
    
    -- Register Options
	--VPCap_Options.args.purge.desc = format(L.core.options.purgeDesc, self:GetPurge() / 24);
	self.aceConfig:RegisterOptionsTable(L.core.name, VPCap_Options, {"VPCap", "TitanVPCap"});
	self.aceConfigDialog:AddToBlizOptions(L.core.name);

    
    -- Events
	self:RegisterEvent("MERCHANT_CLOSED", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_ENTERING_WORLD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("UPDATE_INSTANCE_INFO", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_LEAVING_WORLD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("RAID_INSTANCE_WELCOME", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("LFG_COMPLETION_REWARD", function() VPCap:UpdateLDB(true); end);
	self:RegisterEvent("PLAYER_LEAVE_COMBAT", function() VPCap:UpdateLDB(true); end);
    self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", function() VPCap:UpdateLDB(true); end);

    -- Add to Paperdoll
    table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDVPText);
	table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDVPCapText);
	table.insert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, L.core.PDJPText);
	
	if self:GetVPPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDVPText] = { updateFunc = function(...) VPCap:UpdateVPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPText] = nil;
	end

	if self:GetVPCapPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = { updateFunc = function(...) VPCap:UpdateVPCapPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = nil;
	end

	if self:GetJPPaperdoll() then
		PAPERDOLL_STATINFO[L.core.PDJPText] = { updateFunc = function(...) VPCap:UpdateJPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDJPText] = nil;
	end
	
    -- Build the menu
    self:MenuInitialize();
    
    -- Get working on a score for the player
    self:UpdateLDB(); -- This may cause excesive loading time...
    
end

function VPCap.ldbObj:OnClick(...) 
	VPCap:ShowOptions();
end
 
function VPCap.ldbObj:OnEnter(...)
	local tt;
	local num_columns = 6;
	
	tt = self.LDBtip;
	
	if self.LDBtip == nil then
		tt = VPCap.qtip:Acquire("VPCapTT", num_columns, "LEFT", "RIGHT", "RIGHT", "RIGHT", "RIGHT", "RIGHT");
		self.LDBtip = tt;
	end
	
	--VPCap:UpdateBackdrop();
		
	--tt:SetAutoHideDelay(0.25, self);
	tt:AddHeader(".");
	tt:AddLine(".");
	tt:EnableMouse();
	tt:SmartAnchorTo(self);
	tt:SetScript("OnLeave", nil);
	VPCap:SetTooltipText(tt);
end

function VPCap.ldbObj:OnLeave(...)
	self.LDBtip:Hide();
	VPCap.qtip:Release(self.LDBtip)
	self.LDBtip = nil
end

        --OnTooltipShow = function(tt)
        --                    VPCap:SetTooltipText(tt);
		--				end,


local bgFrame = {insets = {}}
function VPCap:UpdateBackdrop(tt)
	bgFrame.bgFile = LSM:Fetch("background", self.db.global.Background.Texture)
	bgFrame.edgeFile = LSM:Fetch("border", self.db.global.Background.BorderTexture)
	bgFrame.tile = self.db.global.Background.Tile
	bgFrame.tileSize = self.db.global.Background.TileSize
	bgFrame.edgeSize = self.db.global.Background.EdgeSize
	local inset = floor(self.db.global.Background.EdgeSize / 4)
	bgFrame.insets.left = inset
	bgFrame.insets.right = inset
	bgFrame.insets.top = inset
	bgFrame.insets.bottom = inset
	tt:SetBackdrop(bgFrame)

	local c = self.db.global.Background.Color
	tt:SetBackdropColor(c.r, c.g, c.b, c.a)

	c = self.db.global.Background.BorderColor
	tt:SetBackdropBorderColor(c.r, c.g, c.b, c.a)
	--tt:SetBackdropBorderColor(100, 100, 100, 100)
end

function VPCap:SetTooltipText(tt)
    -- Tooltip title
	local val = format(L.core.menuText, format(L.core.versionText, self.version));
	local num_columns, line = 6, 0;
	
	-- determine the number of columns needed
	--if not self:GetCharms() then
	--  num_columns = num_columns - 1;
	--end;
	
	tt = VPCap.LDBtip;
	if tt == nil then 
		tt = VPCap.qtip:Acquire("VPCapTT", num_columns, "LEFT", "RIGHT", "RIGHT", "RIGHT", "RIGHT", "RIGHT");
		VPCap.LDBtip = tt;
	end
	tt:Hide();
	tt:Clear();
	
	VPCap:UpdateBackdrop(tt);
	
	line = tt:AddHeader(".")
	tt:SetCell(line, 1, val, "CENTER", num_columns);
	--tt:SetText(val, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

	-- calculate the time remaining.
	local sec = time();
	local dtDate = time({year=date('%Y',sec),month=date('%m',sec),day=date('%d',sec),hour=0});

	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end
	local dtNext = dtPrev + 7 * 24 * 60 * 60;
 	local timeRemaining = dtNext - sec;

	line = tt:AddSeparator();
	
	line = tt:AddLine(" " .. L.core.cooldownInfoText, ".");
	val = WHITE_BRIGHT_COLOR .. self:GetRemainingTimeText(timeRemaining) .. END_COLOR;
	tt:SetCell(line, 2, val, "LEFT", num_columns-1);
	
	if self:GetDebug() then
		line = tt:AddLine(" Now: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", time()) .. "   " .. time() .. END_COLOR);
		--tt:AddLine("  Last Reset: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtPrev) .. "   " .. dtPrev .. END_COLOR);
		--tt:AddLine("  Date: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtDate) .. "   " .. dtDate .. END_COLOR);
		--tt:AddLine("  Time: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", sec) .. "   " .. sec .. END_COLOR);
		--tt:AddLine("  Resets: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtNext) .. "   " .. dtNext .. END_COLOR);
	end
	line = tt:AddLine(" " .. L.core.cooldownInfoLastText, ".");
	val = WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtPrev) .. END_COLOR;
	tt:SetCell(line, 2, val, "LEFT", num_columns-1);
	
	line = tt:AddLine(" " .. L.core.cooldownInfoNextText, ".");	
	val = WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtNext) .. END_COLOR;
	tt:SetCell(line, 2, val, "LEFT", num_columns-1);
	
	--tt:AddLine("  Date: " .. WHITE_BRIGHT_COLOR  .. date("%d-%b-%Y %I:%M %p", dtDate) .. "   " .. dtDate .. END_COLOR);
	line = tt:AddLine(" ");
	--tt:AddSeparator();
	
	if self:GetWorldBoss() then
		self:LoadAltBoss(tt);
		tt:AddLine(" ");
	end
	
	if self:GetLFR() then
		self:LoadAltLFR(tt);
		tt:AddLine(" ");
	end
	
	self:LoadAlt(tt);

	tt:Show();
	
end

function VPCap:LoadAltBoss(tt)
	local sortorder;

	line = tt:AddHeader(L.core.BossTitle, "Sha", "Galleon", "Nalak", "Oondasta");	
	tt:SetCell(line, 2, L.core.BossShaOfAngerShort, "CENTER")
	tt:SetCell(line, 3, L.core.BossGalleonShort, "CENTER")
	tt:SetCell(line, 4, L.core.BossNalakShort, "CENTER")
	tt:SetCell(line, 5, L.core.BossOondastaShort, "CENTER")
	
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltBoss(toonname, alt, tt);
		end
	
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltBoss(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Valor Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyVP) then
				table.insert(tmp, alt.qtyVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyVP == v then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end
		
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 6 then  -- Justice Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyJP) then
				table.insert(tmp, alt.qtyJP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyJP == v then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 7 then  -- Charms
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyLCharm) then
				table.insert(tmp, alt.qtyLCharm);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyLCharm == v then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 8 then  -- VP Cap Progress
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.periodEarnVP) then
				table.insert(tmp, alt.periodEarnVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.periodEarnVP == v then
					self:ShowAltBoss(toonname, alt, tt);
				end
			end
		end

	end
	
end

function VPCap:ShowAltBoss(toon, alt, tt)
	local toonColor, toonText = "", "";
	
	local sec = time();
	local dtDate = time({year=date('%Y',sec),month=date('%m',sec),day=date('%d',sec),hour=0});
	local boss1, boss2, boss3, boss4;
	
	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end
	dtPrev = dtPrev + 24*60*60; -- add on one day since kill reset is a day after VP reset.
	
	if alt.qtyJP == nil then alt.qtyJP = 0; end
	if alt.qtyVP == nil then alt.qtyVP = 0; end
	
	-- if there is a value for any of the currencies then show the line
	if ((alt.qtyJP > 0 or alt.qtyVP > 0) and (alt.boss1 ~= nil or alt.boss2 ~= nil or alt.boss3 ~= nil or alt.boss4 ~= nil)) or self:GetDebug() then	
		
		-- determine the class color
		for a, b in pairs(CLASS_COLOR) do
			if alt.class == a then
				toonColor = b;
			end
		end
		--local off_left, off_right, off_top, off_bottom = unpack(CLASS_ICON_TCOORDS[alt.class]);
		--local x = off_left .. ":" .. off_top .. ":" .. off_right - off_left+1 .. ":" .. off_bottom-off_top+1;
		--toonText = "|TInterface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes:16:16:" .. x .. "|t" .. toonColor .. toon .. END_COLOR;
		toonText = toonColor .. " " .. toon .. END_COLOR;
		
		if dtPrev < alt.savingTime then
			boss1, boss2, boss3, boss4 = alt.boss1, alt.boss2, alt.boss3, alt.boss4;
		else
			if alt.boss1 ~= nil then boss1 = false end
			if alt.boss2 ~= nil then boss2 = false end
			if alt.boss3 ~= nil then boss3 = false end
			if alt.boss4 ~= nil then boss4 = false end
		end

		line = tt:AddLine(toonText, ".");

		if boss1 ~= nil then 
			if boss1 then 
				boss1Text = RED_NEON_COLOR .. L.core.BossLooted .. END_COLOR
				if self:GetShowIcon() then boss1Text = boss1Text .. " |TInterface\\Icons\\Achievement_pvp_h_06:16:16|t" end
			else 
				boss1Text = GREEN_NEON_COLOR .. L.core.BossNotLooted .. END_COLOR 
				if self:GetShowIcon() then boss1Text = boss1Text .. " |TInterface\\Icons\\Achievement_pvp_g_06:16:16|t" end
			end
			tt:SetCell(line, 2, boss1Text, "CENTER");
		end
		
		if boss2 ~= nil then 
			if boss2 then 
				boss2Text = RED_NEON_COLOR .. L.core.BossLooted .. END_COLOR 
				if self:GetShowIcon() then boss2Text = boss2Text .. " |TInterface\\Icons\\Achievement_pvp_h_06:16:16|t" end
			else 
				boss2Text = GREEN_NEON_COLOR .. L.core.BossNotLooted .. END_COLOR 
				if self:GetShowIcon() then boss2Text = boss2Text .. " |TInterface\\Icons\\Achievement_pvp_g_06:16:16|t" end
			end
			tt:SetCell(line, 3, boss2Text, "CENTER");
		end
		
		if boss3 ~= nil then 
			if boss3 then 
				bossText = RED_NEON_COLOR .. L.core.BossLooted .. END_COLOR 
				if self:GetShowIcon() then boss3Text = boss3Text .. " |TInterface\\Icons\\Achievement_pvp_h_06:16:16|t" end
			else 
				boss3Text = GREEN_NEON_COLOR .. L.core.BossNotLooted .. END_COLOR 
				if self:GetShowIcon() then boss3Text = boss3Text .. " |TInterface\\Icons\\Achievement_pvp_g_06:16:16|t" end
			end
			tt:SetCell(line, 4, boss3Text, "CENTER");
		end
		
		if boss4 ~= nil then 
			if boss4 then 
				boss4Text = RED_NEON_COLOR .. L.core.BossLooted .. END_COLOR 
				if self:GetShowIcon() then boss4Text = boss4Text .. " |TInterface\\Icons\\Achievement_pvp_h_06:16:16|t" end
			else 
				boss4Text = GREEN_NEON_COLOR .. L.core.BossNotLooted .. END_COLOR 
				if self:GetShowIcon() then boss4Text = boss4Text .. " |TInterface\\Icons\\Achievement_pvp_g_06:16:16|t" end
			end
			tt:SetCell(line, 5, boss4Text, "CENTER");
		end
		
			--val = "|TInterface\\Icons\\Misc_arrowdown:16:16|t" --.. "|TInterface\\Icons\\Achievement_pvp_h_06:16:16|t"
			--Achievement_pvp_h_10
			--Misc_arrowdown

			--val = "|TInterface\\Icons\\Misc_arrowlup:16:16|t" --.. "|TInterface\\Icons\\Achievement_pvp_g_06:16:16|t"
			--Achievement_pvp_g_10
			--Misc_arrowlup
	end
	
end

function VPCap:LoadAltLFR(tt)
	local sortorder;

	line = tt:AddHeader(L.core.LFRTitle, "Sha", "Galleon", "Nalak", "Oondasta");	
	tt:SetCell(line, 2, L.core.LFR_MV, "CENTER")
	tt:SetCell(line, 3, L.core.LFR_HoF, "CENTER")
	tt:SetCell(line, 4, L.core.LFR_ToES, "CENTER")
	tt:SetCell(line, 5, L.core.LFR_ToTK, "CENTER")
	
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltLFR(toonname, alt, tt);
		end
	
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAltLFR(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Valor Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyVP) then
				table.insert(tmp, alt.qtyVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyVP == v then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end
		
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 6 then  -- Justice Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyJP) then
				table.insert(tmp, alt.qtyJP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyJP == v then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 7 then  -- Charms
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyLCharm) then
				table.insert(tmp, alt.qtyLCharm);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyLCharm == v then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 8 then  -- VP Cap Progress
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.periodEarnVP) then
				table.insert(tmp, alt.periodEarnVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.periodEarnVP == v then
					self:ShowAltLFR(toonname, alt, tt);
				end
			end
		end

	end
	
end

function VPCap:ShowAltLFR(toon, alt, tt)
	local toonColor, toonText = "", "";
	local cellText = "";
	
	local dungeons = {};
	
	local sec = time();
	local dtDate = time({year=date('%Y',sec),month=date('%m',sec),day=date('%d',sec),hour=0});
	
	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end
	
	-- if the data has not been saved then ignore
	if alt.dungeon == nil then return end
	
	-- if the toon can't do the first raid then ignore
	if not alt.dungeon[LFR_STARTER_DUNGEON].isAvailable then return end
	
	-- determine the class color
	for a, b in pairs(CLASS_COLOR) do
		if alt.class == a then
			toonColor = b;
		end
	end
	toonText = toonColor .. " " .. toon .. END_COLOR;
	
	if dtPrev < alt.savingTime then
		-- use the data in the database
		dungeons = alt.dungeon
	else
		--ignore the data in the database
		for i, v in pairs(alt.dungeon) do
			dungeons[v].isComplete = 0
			dungeons[v].qtyLooted = 0
			dungeons[v].isAvailable = alt.dungeon[v].isAvailable
		end
	end

	line = tt:AddLine(toonText, ".");

	VPCap:SetLFRCell(tt, LFR_CELL_A_DUNGEONS, dungeons, line, 2)
	VPCap:SetLFRCell(tt, LFR_CELL_B_DUNGEONS, dungeons, line, 3)
	VPCap:SetLFRCell(tt, LFR_CELL_C_DUNGEONS, dungeons, line, 4)
	--VPCap:SetLFRCell(tt, LFR_CELL_D_DUNGEONS, dungeons, line, 5)
	
end

function VPCap:LoadAlt(tt)

	local sortorder;

	line = tt:AddHeader(L.core.PDCharTextShort);
	if self:GetiLevel() then
		tt:SetCell(line, 2, L.core.PDiLvlTextShort, "CENTER")
	end
	tt:SetCell(line, 3, L.core.PDVPCapTextShort, "CENTER")
	tt:SetCell(line, 4, L.core.PDVPTextShort, "CENTER")
	tt:SetCell(line, 5, L.core.PDJPTextShort, "CENTER")
	if self:GetCharms() then 
		tt:SetCell(line, 6, L.core.PDCharmTextShort, "CENTER")
	end
		
	sortorder = self:GetSortOrder();
	
	if sortorder == 1 then  -- Unsorted
	
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAlt(toonname, alt, tt);
		end
	
	elseif sortorder == 2 then  -- Alphabetical
		for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
			self:ShowAlt(toonname, alt, tt);
		end
	
	elseif sortorder == 3 then  -- Class
		for class in orderedPairs(CLASS_COLOR) do
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.class == class then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end
	
	elseif sortorder == 4 then  -- Valor Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyVP) then
				table.insert(tmp, alt.qtyVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyVP == v then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end
		
		
	elseif sortorder == 5 then  -- iLevel
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.iLvl) then
				table.insert(tmp, alt.iLvl);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.iLvl == v then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 6 then  -- Justice Points
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyJP) then
				table.insert(tmp, alt.qtyJP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyJP == v then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 7 then  -- Charms
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.qtyLCharm) then
				table.insert(tmp, alt.qtyLCharm);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.qtyLCharm == v then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end

	elseif sortorder == 8 then  -- VP Cap Progress
		tmp = {};
		for toonname, alt in pairs(TitanVPCap_Storage[self.currentRealm]) do 
			if not table.contains(tmp, alt.periodEarnVP) then
				table.insert(tmp, alt.periodEarnVP);
			end
		end
		table.sort(tmp, function(a,b) return a>b end);	
		for i, v in ipairs(tmp) do 
			for toonname, alt in orderedPairs(TitanVPCap_Storage[self.currentRealm]) do
				if alt.periodEarnVP == v then
					self:ShowAlt(toonname, alt, tt);
				end
			end
		end

	end
	
end

function VPCap:ShowAlt(toon, alt, tt)
	local toonColor, toonText, valCapColor, valVPColor, valJPColor, valECharmColor, valCapText, valCharmText, valCharmColor = "", "", "", "", "", "", "", "", "";
	local valVP, valJP, valCap, valiLvl, valiELvl, valECharm, valLCharm = 0,0,0,0,0,0, 0;

	local sec = time();
	local dtDate = time({year=date('%Y',sec),month=date('%m',sec),day=date('%d',sec),hour=0});

	local dtPrev = dtDate - (((date("%w", dtDate) - self.db.global.reset_dayofweek) % 7) ) * 86400 + (self.db.global.reset_hour * 60 * 60);
	if dtPrev > sec then
	  dtPrev = dtPrev - (7 * 24 * 60 * 60);
	end

	-- if there is a value for any of the currencies then show the line
	if alt.qtyVP > 0 or alt.qtyJP > 0 or alt.qtyECharm > 0 or alt.qtyLCharm > 0 or self:GetDebug() then

		if alt.qtyMaxVP == nil then
			alt.qtyMaxVP = self.db.global.VPStandardMax;
		elseif alt.qtyMaxVP <= 0 then
			alt.qtyMaxVP = self.db.global.VPStandardMax;
		end 

		if alt.periodMaxVP == nil then
			alt.periodMaxVP = self.db.global.VPStandardCapMax;
		elseif alt.periodMaxVP <= 0 then
			alt.periodMaxVP = self.db.global.VPStandardCapMax;
		end 

		if alt.qtyMaxJP == nil then
			alt.qtyMaxJP = self.db.global.JPStandardMax;
		elseif alt.qtyMaxJP <= 0 then
			alt.qtyMaxJP = self.db.global.JPStandardMax;
		end 
		
		if alt.iLvl == nil then alt.iLvl = 0 end
		if alt.iELvl == nil then alt.iELvl = 0 end		
		if alt.qtyECharm == nil then alt.qtyECharm = 0 end
		if alt.qtyLCharm == nil then alt.qtyLCharm = 0 end
	
		self:Debug("VP Max Cap: " .. alt.qtyMaxVP);
	
		-- determine the class color
		for a, b in pairs(CLASS_COLOR) do
			if alt.class == a then
				toonColor = b;
			end
		end
		--local off_left, off_right, off_top, off_bottom = unpack(CLASS_ICON_TCOORDS[alt.class]);
		--local x = off_left .. ":" .. off_top .. ":" .. off_right - off_left+1 .. ":" .. off_bottom-off_top+1;
		--toonText = "|TInterface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes:16:16:" .. x .. "|t" .. toonColor .. toon .. END_COLOR;
		toonText = toonColor .. toon .. END_COLOR;
		
		if dtPrev < alt.savingTime then
			valCap = alt.periodEarnVP;
		else
			valCap = 0;
		end

		valiLvl = alt.iLvl;
		valiELvl = alt.iELvl;
		valVP = alt.qtyVP;
		valJP = alt.qtyJP;
		valECharm = alt.qtyECharm;
		valLCharm = alt.qtyLCharm;

		valCapColor = self:GetProgressColor(valCap, alt.periodMaxVP);
		valVPColor = self:GetProgressColor(valVP, alt.qtyMaxVP);
		valJPColor = self:GetProgressColor(valJP, alt.qtyMaxJP);
		valCharmColor = GREEN_NEON_COLOR;
		if self:GetiLevel() then
			valiLvlText = valiLvl 
			if self:GetShowIcon() then valiLvlText = valiLvlText .. " |TInterface\\Icons\\Inv_helmet_03:16:16|t" end
		else
			valiLvlText = nil
		end
		valCapText = valCapColor .. valCap .. END_COLOR -- .. "/" .. alt.periodMaxVP .. END_COLOR
		if self:GetShowIcon() then valCapText = valCapText .. " |TInterface\\Icons\\pvecurrency-valor:16:16|t" end
		valVPText = valVPColor .. valVP .. END_COLOR 
		if self:GetShowIcon() then valVPText = valVPText .. " |TInterface\\Icons\\pvecurrency-valor:16:16|t" end
		valJPText = valJPColor .. valJP .. END_COLOR 
		if self:GetShowIcon() then valJPText = valJPText .. " |TInterface\\Icons\\pvecurrency-justice:16:16|t" end
		if self:GetCharms() then
			valCharmText = valCharmColor .. valLCharm .. "/" .. valECharm .. END_COLOR 
			if self:GetShowIcon() then valCharmText = valCharmText .. " |TInterface\\Icons\\inv_misc_coin_17:16:16|t" end
		else
			valCharmText = nil
		end
-- lesser charm icon inv_misc_coin_18
-- elder charm icon inv_misc_coin_17

		if self:GetDebug() then
			tt:AddLine(" " .. toonText,  valVPColor .. 1234 .. END_COLOR, valCapColor .. "1234/1234" .. END_COLOR);
		else
			tt:AddLine(" " .. toonText, valiLvlText, valCapText , valVPText, valJPText, valCharmText);
		end
	end
end

function VPCap:SetLFRCell(tt, dungeonList, data, line, cellID)

	cellText = ""
	for i, v in orderedPairs(dungeonList) do
		if cellText ~= "" then cellText = cellText .. " " end
		if data[v].isAvailable == 1 then
			if data[v].isComplete == 1 then
				-- red text
				cellText = cellText .. RED_NEON_COLOR
			else
				-- green text
				cellText = cellText .. GREEN_NEON_COLOR
			end
		else
			-- orange text
			cellText = cellText .. ORANGE_NEON_COLOR
		end
		cellText = cellText .. data[v].qtyLooted .. END_COLOR
	end
	tt:SetCell(line, cellID, cellText, "CENTER");

end

function VPCap:GetProgressColor(val, maxval)
	if val >= maxval then 
		return RED_NEON_COLOR;
	elseif val > (maxval - self.db.global.progress_orange) then 
		return ORANGE_NEON_COLOR;
	elseif val > (maxval - self.db.global.progress_yellow) then 
		return YELLOW_NEON_COLOR;
	else 
		return GREEN_NEON_COLOR;
	end
	return GREEN_NEON_COLOR;
end

-- Save the data to the database.
function VPCap:SaveData()

	local qtyVP, qtyMaxVP, wk_earnVP, wk_MaxVP
	local qtyJP, wk_earnJP, qtyMaxJP
	local qtyECharm, qtyLCharm, qtyMCharm
	local iLvl, iELvl
	local a, b, c, d, e, f
	local data
	
	self:Debug("Saving VP Cap...");
	self:Debug("Current Player " .. self.currentPlayer);
	self:Debug("Current Realm " .. self.currentRealm);
	
	if TitanVPCap_Storage == nil then
		TitanVPCap_Storage = {};
	end
	if TitanVPCap_Storage[self.currentRealm] == nil then
		TitanVPCap_Storage[self.currentRealm] = {};
	end	
	if TitanVPCap_Storage[self.currentRealm][self.currentPlayer] == nil then
		TitanVPCap_Storage[self.currentRealm][self.currentPlayer] = {};
		data = TitanVPCap_Storage[self.currentRealm][self.currentPlayer];
		local playerClass, englishClass = UnitClass("player");
		data.class = englishClass;
	else 
		data = TitanVPCap_Storage[self.currentRealm][self.currentPlayer];
	end
	data.savingTime = time();
		
	a, qtyVP, b, wk_earnVP, wk_MaxVP, qtyMaxVP, e = GetCurrencyInfo(VALOR_CURRENCY_ID);
	a, qtyJP, b, wk_earnJP, c, qtyMaxJP, e = GetCurrencyInfo(JUSTICE_CURRENCY_ID);
	a, qtyECharm, b, c, d, e, f = GetCurrencyInfo(ELDER_CHARM_CURRENCY_ID);
	a, qtyLCharm, b, c, d, e, f = GetCurrencyInfo(LESSER_CHARM_CURRENCY_ID);
	a, qtyMCharm, b, c, d, e, f = GetCurrencyInfo(MOGU_CHARM_CURRENCY_ID);
	iLvl, iELvl = GetAverageItemLevel();
	
	wk_MaxVP = floor(wk_MaxVP/100);
	qtyMaxVP = floor(qtyMaxVP/100);
	qtyMaxJP = floor(qtyMaxJP/100);
	iLvl = floor(iLvl);
	iELvl = floor(iELvl);
	
	-- save data 
	-- New values that are recorded.
	data.qtyVP = qtyVP;
	data.qtyMaxVP = qtyMaxVP;
	data.periodEarnVP = wk_earnVP;
	data.periodMaxVP = wk_MaxVP;
	
	data.qtyJP = qtyJP;
	data.periodEarnJP = wk_earnJP; 
	data.qtyMaxJP = qtyMaxJP;
	
	data.qtyECharm = qtyECharm;
	data.qtyLCharm = qtyLCharm;
	data.qtyMCharm = qtyMCharm;
	
	if iLvl ~= 0 then
		data.iLvl = iLvl;
	end
	
	if iELvl ~= 0 then
		data.iELvl = iELvl;
	end

	-- save off the LFR Raid saves/completions
				
	for i, v in pairs(LFR_DUNGEONIDS) do
		_, numKilled = GetLFGDungeonNumEncounters(v);
		_, _, raidCompleted, raidAvailable = GetLFGDungeonRewardCapInfo(v);
	
		if data.dungeon == nil then data.dungeon = {} end
		if data.dungeon[v] == nil then data.dungeon[v] = {} end
		data.dungeon[v].qtyLooted = numKilled
		data.dungeon[v].isComplete = raidCompleted
		data.dungeon[v].isAvailable = raidAvailable
	end

	-- reset the values to future proof the addon
	data.boss1 = nil;
	data.boss2 = nil;
	data.boss3 = nil;
	data.boss4 = nil;
	
	-- save off the boss kills of interest.
	--local z,t={[32099]=L.core.BossShaOfAnger,[32098]=L.core.BossGalleon,[32518]=L.core.BossNalak,[32519]=L.core.BossOondasta},GetQuestsCompleted();
	local z,t={[32099]=L.core.BossShaOfAnger,[32098]=L.core.BossGalleon},GetQuestsCompleted();
	for c,v in pairs(z) do 
		if c == 32099 then -- Sha of Anger
			if t[c] == nil then
				data.boss1 = false;
			else
				data.boss1 = t[c];
			end
		elseif c == 32098 then -- Galleon
			if t[c] == nil then
				data.boss2 = false;
			else
				data.boss2 = t[c];
			end
		elseif c == 32518 then -- Nalak
			if t[c] == nil then
				data.boss3 = false;
			else
				data.boss3 = t[c];
			end
		elseif c == 32519 then -- Oondasta
			if t[c] == nil then
				data.boss4 = false;
			else
				data.boss4 = t[c];
			end
		end
		
	end
	
	
end


-- Make sure the database is the latest version
function VPCap:UpdateSettings()
    
    if self.db.global.version == self.versionMajor then
        -- Same version
    end
    
    -- Update version information
    self.db.global.version = self.versionMajor;
end

-- Purge the Database
function VPCap:PurgeDB()

	TitanVPCap_Storage = nil;
	
	self:SaveData();

end

--[[

    Setters, Getter and Togglers

]]

-- Set
function VPCap:SetLabel(v) self.db.global.showLabel = v; self:UpdateLDB(); end
function VPCap:SetLDBlabel(v) self.db.global.ldbLabel = v; self:UpdateLDB(true); end
function VPCap:SetLDBTextColor(v) self.db.global.ldbTextColor = v; self:UpdateLDB(true); end
function VPCap:SetCharms(v) self.db.global.charms = v; end
function VPCap:SetiLevel(v) self.db.global.iLevel = v; end
function VPCap:SetSortOrder(v) self.db.global.sortOrder = v; end
function VPCap:SetWorldBoss(v) self.db.global.worldBoss = v; end
function VPCap:SetShowIcon(v) self.db.global.show_Icons = v; end
function VPCap:SetLFR(v) self.db.global.lfr = v; end
function VPCap:SetBGGeneric(i, v) self.db.global.Background[ i[#i] ] = v end

-- Get
function VPCap:GetMinimap() return not self.db.global.minimap.hide; end
function VPCap:GetVPPaperdoll() return self.db.global.cinfoVP; end
function VPCap:GetVPCapPaperdoll() return self.db.global.cinfoVPCap; end
function VPCap:GetJPPaperdoll() return self.db.global.cinfoJP; end
function VPCap:GetLabel() return self.db.global.showLabel; end
function VPCap:GetLDB() return self.db.global.ldbText; end
function VPCap:GetLDBTextColor() return self.db.global.ldbTextColor; end
function VPCap:GetLDBlabel() return self.db.global.ldbLabel; end
function VPCap:GetDebug() return self.db.char.debug; end
function VPCap:GetCharms() return self.db.global.charms; end
function VPCap:GetiLevel() return self.db.global.iLevel; end
function VPCap:GetSortOrder() return self.db.global.sortOrder; end
function VPCap:GetWorldBoss() return self.db.global.worldBoss; end
function VPCap:GetShowIcon() return self.db.global.show_Icons; end
function VPCap:GetBGGeneric(i) return self.db.global.Background[ i[#i] ] end
function VPCap:GetBGTile() return self.db.global.Background.Tile end
function VPCap:GetLFR() return self.db.global.lfr end

-- Toggle
function VPCap:ToggleShowIcon() self:SetShowIcon(not self:GetShowIcon()); end
function VPCap:ToggleMinimap() self:SetMinimap(not self:GetMinimap()); end
function VPCap:ToggleCharms() self:SetCharms(not self:GetCharms()); end
function VPCap:ToggleiLevel() self:SetiLevel(not self:GetiLevel()); end
function VPCap:ToggleWorldBoss() self:SetWorldBoss(not self:GetWorldBoss()); end
function VPCap:ToggleSortOrder() self:SetSortOrder(not self:GetSortOrder()); end
function VPCap:ToggleVPPaperdoll() self:SetVPPaperdoll(not self:GetVPPaperdoll()); end
function VPCap:ToggleVPCapPaperdoll() self:SetVPCapPaperdoll(not self:GetVPCapPaperdoll()); end
function VPCap:ToggleJPPaperdoll() self:SetJPPaperdoll(not self:GetJPPaperdoll()); end
function VPCap:ToggleLabel() self:SetLabel(not self:GetLabel()); end
function VPCap:ToggleLDBlabel() self:SetLDBlabel(not self:GetLDBlabel()); end
function VPCap:ToggleLDBTextColor() self:SetLDBTextColor(not self:GetLDBTextColor()); end
function VPCap:ToggleLFR() self:SetLFR(not self:GetLFR()); end

-- Advanced Gets
function VPCap:GetBGColor(i)
	local t = self.db.global.Background.Color
	return t.r, t.g, t.b, t.a
end

function VPCap:GetBGBorderColor(i)
	local x = self.db.global.Background.BorderColor
	return x.r, x.g, x.b, x.a
end

function VPCap:GetRemainingTimeText(seconds)
	local days = floor(seconds / 86400);
	seconds = seconds - days * 86400;
	local hours = floor(seconds / 3600);
	seconds = seconds - hours * 3600;
	local minutes = floor(seconds / 60);
	seconds = seconds - minutes * 60;
	local TIME_COLOR = "";
	local TIME_TEXT = "";
	local timeStr = "";		-- one space just to pad slightly vs being too close to raid/instance name string.
	
	-- compact view's colors.
	-- moved "Reset in" text so that it's now above lockout time.
	TIME_COLOR = WHITE_BRIGHT_COLOR ;
	TIME_TEXT = WHITE_BRIGHT_COLOR ;
	
	if days > 0 then
		timeStr = timeStr .. TIME_COLOR .. days .. TIME_TEXT .. " " .. L.core.cooldownInfoDay .. END_COLOR .. " ";
	end
	
	if hours > 0 then
		timeStr = timeStr .. TIME_COLOR .. hours .. TIME_TEXT .. " " .. L.core.cooldownInfoHr .. END_COLOR .. " ";
	end

	if minutes then 
		timeStr = timeStr .. TIME_COLOR .. minutes .. TIME_TEXT .. " " .. L.core.cooldownInfoMin .. END_COLOR .. " ";
	end


	return timeStr;
end

function VPCap:GetButtonText()
	local a = " ";
	local val;
	local earnVP = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP;

	if self:GetPeriodMaxVP() <= 0 then
		val = "N/A";
	else
		val = earnVP .. "/" .. self:GetPeriodMaxVP();
	end

	self:Debug("PeriodMaxVP = " .. self:GetPeriodMaxVP());
	self:Debug("Earnt VP = " .. earnVP);
	
	if earnVP >= self:GetPeriodMaxVP() then 
		a = RED_NEON_COLOR;
	elseif earnVP > (self:GetPeriodMaxVP() - self.db.global.progress_orange) then 
		a = ORANGE_NEON_COLOR;
	elseif earnVP > (self:GetPeriodMaxVP() - self.db.global.progress_yellow) then 
		a = YELLOW_NEON_COLOR;
	else
		a = GREEN_NEON_COLOR;
	end

	if a then
	  self:Debug("col at line 405 has a value");
	else
	  self:Debug("col does not have a value");
	end 
	
	if self:GetLDBTextColor() then
		return " " .. a .. val .. END_COLOR;
	else
		return " " .. val;
	end
	
end

function VPCap:GetMaxVP()
	local a, b, c, d
	
	if self.maxVP >= 0 then
		return self.maxVP;
	else
		a, b, c, d, self.periodMaxVP, self.maxVP = GetCurrencyInfo(VALOR_CURRENCY_ID);
		self.periodMaxVP = floor(self.periodMaxVP / 100);
		self.maxVP = floor(self.maxVP/100);
		return self.maxVP;
	end

end

function VPCap:GetMaxJP()
	local a, b, c, d, e
	
	if self.maxJP >= 0 then
		return self.maxJP;
	else
		a, b, c, d, e, self.maxJP = GetCurrencyInfo(JUSTICE_CURRENCY_ID);
		self.maxJP = floor(self.maxJP/100);
		return self.maxJP;
	end

end

function VPCap:GetPeriodMaxVP()
	local a, b, c, d
	
	if self.periodMaxVP > 0 then
		return self.periodMaxVP;
	else
		a, b, c, d, self.periodMaxVP, self.maxVP = GetCurrencyInfo(VALOR_CURRENCY_ID);
		self.periodMaxVP = floor(self.periodMaxVP / 100);
		self.maxVP = floor(self.maxVP/100);
		
		return self.periodMaxVP;
	end

end

function VPCap:GetResetHour() 
	if self.db.global.reset_hour < 0 or self.db.global.reset_hour > 23 then
		self.db.global.reset_hour = VPCap_Defaults.global.reset_hour;
	end
	
	return self.db.global.reset_hour; 
	
end

function VPCap:GetResetDayOfWeek() 
	if self.db.global.reset_dayofweek < 1 or self.db.global.reset_dayofweek > 7 then
		self.db.global.reset_dayofweek = VPCap_Defaults.global.reset_dayofweek;
	end
	
	return self.db.global.reset_dayofweek; 

end


-- Advanced sets
function VPCap:SetBGColor(r, g, b, a)
	local t = self.db.global.Background.Color
	t.r, t.g, t.b, t.a = r, g, b, a
end

function VPCap:SetBGBorderColor(r, g, b, a)
	local x = self.db.global.Background.BorderColor
	x.r, x.g, x.b, x.a = r, g, b, a
end

function VPCap:SetMinimap(v) 
    self.db.global.minimap.hide = not v;
	
	if not v then
		self.ldbIcon:Hide(L.core.name);
	else
		self.ldbIcon:Show(L.core.name);
	end
end

function VPCap:SetResetHour(v) 
	if v < 0 or v > 23 then
		self.db.global.reset_hour = VPCap_Defaults.global.reset_hour;
	else
		self.db.global.reset_hour = v; 
	end
	
end

function VPCap:SetResetDayOfWeek(v) 
	if v < 1 or v > 7 then
		self.db.global.reset_dayofweek = VPCap_Defaults.global.reset_dayofweek;
	else
		self.db.global.reset_dayofweek = v;
	end 

end

function VPCap:SetDebug(v) 

	self.db.char.debug = v; 
	
	if v then 
		self:Print("Entering debug mode");
	else 
		self:Print("Leaving debug mode");
	end

end

function VPCap:SetLDB(v)
	self.db.global.ldbText = v;
	
	if v then
		self.ldb.type = 'data source';
	else
		self.ldb.type = 'launcher';
		self.ldb.text = nil;
	end
	
	self:UpdateLDB(true);
end

function VPCap:SetVPPaperdoll(v)
	self.db.global.cinfoVP = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDVPText] = { updateFunc = function(...) VPCap:UpdateVPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPText] = nil;
	end
end

function VPCap:SetVPCapPaperdoll(v)
	self.db.global.cinfoVPCap = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = { updateFunc = function(...) VPCap:UpdateVPCapPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDVPCapText] = nil;
	end
end

function VPCap:SetJPPaperdoll(v)
	self.db.global.cinfoJP = v;
	
	if v then
		PAPERDOLL_STATINFO[L.core.PDJPText] = { updateFunc = function(...) VPCap:UpdateJPPaperDollFrame(...); end };
	else
		PAPERDOLL_STATINFO[L.core.PDJPText] = nil;
	end
end


-- Advanced Updates
function VPCap:UpdateLDB(force, auto)
	
	self:SaveData();
	
	if self:GetLDB() then
		local label = UnitName('player');
		
		-- Do we really need to update LDB?
		if force or label ~= self.ldbLabel then
            
            local text = self:GetButtonText();
			self:UpdateLDBText(label, text)

		end
	else
		self.ldb.type = 'launcher';
		self.ldb.text = nil;
	end
end

function VPCap:UpdateLDBText(label, text)
    
	if not self:GetLDB() then return false; end
    
    -- Add the label
    if self:GetLDBlabel() then
        text = label..": "..text;
    end
    
    self.ldb.text = text;
    self.ldbUpdated = time();
    self.ldbLabel = label;
end

function VPCap:UpdateVPPaperDollFrame(statFrame, unit)
    
	local formatted;
	
	self:SaveData();
	
	PaperDollFrame_SetLabelAndText(statFrame, L.core.PDVPText, TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyVP, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDVPText..FONT_COLOR_CODE_CLOSE;
    if self:GetMaxVP() > 0 then
	  formatted = L.core.PDVPTooltip .. "\n" .. format(L.core.PDVPTooltipMax, self:GetMaxVP());
	else
	  formatted = L.core.PDVPTooltip;
    end
    statFrame.tooltip2 = formatted;
	
    statFrame:Show();
end

function VPCap:UpdateVPCapPaperDollFrame(statFrame, unit)
    
	local formatted;
	
	if self:GetPeriodMaxVP() > 0 then
	  formatted = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP .. "/" .. self:GetPeriodMaxVP();
	else
	  formatted = TitanVPCap_Storage[self.currentRealm][self.currentPlayer].periodEarnVP;
	end
	
    PaperDollFrame_SetLabelAndText(statFrame, L.core.PDVPCapText, formatted, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDVPCapText..FONT_COLOR_CODE_CLOSE;
    if self:GetPeriodMaxVP() > 0 then
	  formatted = L.core.PDVPCapTooltip .. "\n" .. format(L.core.PDVPCapTooltipMax, self:GetPeriodMaxVP());
	else
	  formatted = L.core.PDVPCapTooltip;
    end
    statFrame.tooltip2 = formatted;
    
    statFrame:Show();
end

function VPCap:UpdateJPPaperDollFrame(statFrame, unit)
    	
    PaperDollFrame_SetLabelAndText(statFrame, L.core.PDJPText, TitanVPCap_Storage[self.currentRealm][self.currentPlayer].qtyJP, false);
    statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..L.core.PDJPText..FONT_COLOR_CODE_CLOSE;
    if self:GetMaxJP() > 0 then
	  formatted = L.core.PDJPTooltip .. "\n" .. format(L.core.PDJPTooltipMax, self:GetMaxJP());
	else
	  formatted = L.core.PDJPTooltip;
    end
    statFrame.tooltip2 = formatted;
	
    statFrame:Show();
end

-- Debug function
function VPCap:Debug(...)
	if self:GetDebug() then
		print('|cFFFF0000TitanVPCap Debug:|r ', ...);
	end
end

-- Open the options window
function VPCap:ShowOptions()
    -- Not using this until a paramiter is added to open expanded
    -- InterfaceOptionsFrame_OpenToCategory(L.core.name);
    
	self:SaveData();
	
    for i,element in pairs(INTERFACEOPTIONS_ADDONCATEGORIES) do 
        if element.name == L.core.name then
            if not InterfaceOptionsFrame:IsShown() then
                InterfaceOptionsFrame_Show();
                InterfaceOptionsFrameTab2:Click();
            end

            for i,button in pairs(InterfaceOptionsFrameAddOns.buttons) do
                if button.element == element then
                    InterfaceOptionsListButton_OnClick(button);
                    
                    -- Expand
                    if element.hasChildren and element.collapsed then
                        OptionsListButtonToggle_OnClick(button.toggle);
                    end
                
                -- Hide anything else that is open
                elseif button.element and type(button.element.collapsed) == 'boolean' and not button.element.collapsed then
                    OptionsListButtonToggle_OnClick(button.toggle);
                end
            end
        end
    end
end

